/*
* BSD License
*
* Copyright (c) 2007, The University of Manchester (UK)
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*     - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     - Redistributions in binary form must reproduce the above
*       copyright notice, this list of conditions and the following
*       disclaimer in the documentation and/or other materials provided
*       with the distribution.
*     - Neither the name of the University of Manchester nor the names
*       of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written
*       permission.

* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Threading;

/// <summary> </summary>
/// <author>  ansarim
/// </author>
public class LeeThread
{
    private static ThreadState[] threadsStates = new ThreadState[100];
    private int threadID = -1;

    public ThreadState ThreadLocalThreadState
    {
        get
        {
            if (threadID == -1)
                threadID = int.Parse(Thread.CurrentThread.Name);
            if (threadsStates[threadID] == null)
                threadsStates[int.Parse(Thread.CurrentThread.Name)] = new ThreadState();
            return threadsStates[threadID];
        }

        set
        {
            if (threadID == -1)
                threadID = int.Parse(Thread.CurrentThread.Name);
            threadsStates[int.Parse(Thread.CurrentThread.Name)] = value;
        }
    }

    /// <summary> Class that holds thread's actual state</summary>
    public class ThreadState
    {
        public long myLaidTracks = 0; // number of laid tracks

        /// <summary> Creates new ThreadState</summary>
        public ThreadState()
        {
        }

        /// <summary> Resets any metering information (commits/aborts, etc).</summary>
        public virtual void reset()
        {
            myLaidTracks = 0; // total number of transactions
        }

        /// <summary> used for debugging</summary>
        /// <returns> string representation of thread state
        /// </returns>
        public override System.String ToString()
        {
            return "Thread" + GetHashCode() + "[" + "total: " + myLaidTracks + "," + "]";
        }
    }

    public static bool stop = false;
    internal bool finished = false;
    public bool sampleNow = false;
    public bool doneSample = true;
    // public static long totalTracks=0;
    public static long totalLaidTracks = 0;
    public long myLaidTracks = 0;
    private static System.Object lock_Renamed = new System.Object();
    public System.String hardware = System.Environment.GetEnvironmentVariable("HOSTNAME");
    // protected static ThreadLocal_ThreadState _threadState = new ThreadLocal_ThreadState();

    //static ThreadLocal_Thread _thread = new ThreadLocal_Thread();


    LeeRouter lt;
    WorkQueue t;
    bool done = true;
    int[, ,] tempg;

    public LeeThread(LeeRouter lt)
    {
        this.lt = lt;
        tempg = new int[lt.GRID_SIZE, lt.GRID_SIZE, 2]; // Lee 2D Grid copy
    }

    public void run()
    {
        while (!finished && !stop)
        {
            if (sampleNow)
            {
                collectMyStatistics();
                doneSample = true;
                sampleNow = false;
            }
            if (done)
            {
                t = lt.NextTrack;
                done = false;
            }
            if (t == null)
            {
                finished = true;
                Console.WriteLine("Finished");
                collectMyStatistics();
                collectStatistics(this.ThreadLocalThreadState);
                break;
            }
            else
            {
                for (int x = 0; x < lt.GRID_SIZE; x++)
                {
                    for (int y = 0; y < lt.GRID_SIZE; y++)
                    {
                        for (int z = 0; z < 2; z++)
                            tempg[x, y, z] = LeeRouter.TEMP_EMPTY;
                    }
                }
                // System.Console.Out.WriteLine("Laying track "+t.nn);
                lt.layNextTrack(t, tempg);
                done = true;
                updateStatistics();
            }
        }
    }


    protected static void collectStatistics(ThreadState threadState)
    {
        // collect statistics
        lock (lock_Renamed)
        {
            totalLaidTracks += threadState.myLaidTracks;
            threadState.reset();  // set up for next iteration
        }
    }

    public void updateStatistics()
    {
        this.ThreadLocalThreadState.myLaidTracks++;
    }

    public void collectMyStatistics()
    {
        myLaidTracks = this.ThreadLocalThreadState.myLaidTracks - myLaidTracks;
    }

    public void resetMyStatistics()
    {
        myLaidTracks = 0;
    }
}